import unittest, pprint
from simpleparse.simpleparsegrammar import generator
try:
	from TextTools import TextTools
except ImportError:
	from mx.TextTools import TextTools

class SimpleParseGrammarTests(unittest.TestCase):
	def doBasicTest(self, parserName, testValue, expected, ):
		parser = generator.buildParser( parserName )
		result = TextTools.tag( testValue, parser )
		assert result == expected, '''\n\texpected:%s\n\tgot:%s\n'''%( expected, result )
	def testChar1( self ):
		self.doBasicTest(
			"CHARNODBLQUOTE",
			'test\\""',
			(1, [], 4),
		)
	def testChar2( self ):
		self.doBasicTest(
			"ESCAPEDCHAR",
			'\\n"',
			(1, [('SPECIALESCAPEDCHAR', 1, 2, [])], 2),
		)
	def testChar3( self ):
		self.doBasicTest(
			"ESCAPEDCHAR",
			'\\007"',
			(1, [('OCTALESCAPEDCHAR', 1, 4, [])], 4),
		)
	def testChar4( self ):
		testValue = '\\""'
		print 'testChar4', repr(testValue)
		self.doBasicTest(
			"CHARNODBLQUOTE",
			testValue,
			(0, [], 0),
		)
	def testChar4( self ):
		self.doBasicTest(
			"CHARNODBLQUOTE",
			'ehllo\\""',
			(1, [], 5),
		)
	def testChar6( self ):
		self.doBasicTest(
			"CHARNODBLQUOTE",
			'007',
			(1, [], 3),
		)
	def testRange1( self ):
		self.doBasicTest(
			"range",
			'[a-zA-Z]',
			(1, [('CHARNOBRACE', 1, 7, [('CHAR', 1, 7, [])])], 8),
		)
	def testRange2( self ):
		self.doBasicTest(
			"range",
			'[-a-zA-Z]',
			(1, [
				('CHARDASH', 1, 2, []),
				('CHARNOBRACE', 2, 8, [
					('CHAR', 2, 8,[])
				])
			], 9),
		)
	def testRange3( self ):
		self.doBasicTest(
			"range",
			'[]a-zA-Z]',
			(1, [
				('CHARBRACE', 1, 2, []),
				('CHARNOBRACE', 2, 8, [
					('CHAR', 2, 8,[])
				])
			], 9),
		)
# Major failure here, system hangs trying to parse the [] string :( ...				
##	def testRange4( self ):
##		self.doBasicTest(
##			"range",
##			'[]',
##			(0, [], 0),
##		)

		
	def testLiteral1( self ):
		self.doBasicTest(
			"literal",
			'"test"',
			(1, [('CHARNODBLQUOTE', 1, 5, [])], 6),
		)
	def testLiteral2( self ):
		self.doBasicTest(
			"literal",
			'"test\\""',
			(1, [
				('CHARNODBLQUOTE', 1, 5, []),
				('ESCAPEDCHAR', 5, 7, [
					('SPECIALESCAPEDCHAR', 6, 7, [])
				])
			], 8)
			
		)
	def testLiteral3( self ):
		self.doBasicTest(
			"literal",
			'""',
			(1, [], 2),
		)
	def testLiteral4( self ):
		self.doBasicTest(
			"literal",
			'"\'"',
			(1, [('CHARNODBLQUOTE', 1, 2, []),], 3),
		)
	def testLiteral5( self ):
		self.doBasicTest(
			"literal",
			'"\\"test"',
			(1, [
				('ESCAPEDCHAR', 1, 3, [
					('SPECIALESCAPEDCHAR', 2, 3, [])
				]),
				('CHARNODBLQUOTE', 3, 7, [])
			], 8)			
		)
		
	def testWhitespace1( self ):
		self.doBasicTest(
			"ts",
			'  \t',
			(1, [], 3)
		)
	def testWhitespace2( self ):
		self.doBasicTest(
			"ts",
			'  \t\n',
			(1, [], 4)
		)
	def testWhitespace3( self ):
		self.doBasicTest(
			"ts",
			'  \t#testing\r\n',
			(1, [('comment', 3, 13, [])], 13)
		)
	def testWhitespace4( self ):
		self.doBasicTest(
			"ts",
			'nospace',
			(1, [], 0)
		)
		
	def testName1( self ):
		self.doBasicTest(
			"name",
			'abcdefg',
			(1, [], 7)
		)
	def testName2( self ):
		self.doBasicTest(
			"name",
			'2abcdefg',
			(0, [], 0)
		)
	def testName3( self ):
		self.doBasicTest(
			"name",
			'_abcdefg_-',
			(1, [], 9)
		)

	def testUnreportedName1( self ):
		self.doBasicTest(
			"unreportedname",
			'<abcdefg>',
			(1, [('name',1,8,[])], 9)
		)
	def testUnreportedName2( self ):
		self.doBasicTest(
			"unreportedname",
			'<>',
			(0, [], 0)
		)
	def testComment1( self ):
		self.doBasicTest(
			"comment",
			'>',
			(0, [], 0)
		)
	def testComment2( self ):
		self.doBasicTest(
			"comment",
			'#testing\n',
			(1, [], 9)
		)

		
def getSuite():
	return unittest.makeSuite(SimpleParseGrammarTests,'test')

if __name__ == "__main__":
	unittest.main(defaultTest="getSuite")
